@@ -1,6 +1,6 @@
 package Git::Raw::Tag;
 {
-  $Git::Raw::Tag::VERSION = '0.20';
+  $Git::Raw::Tag::VERSION = '0.21';
 }
 
 use strict;
@@ -14,7 +14,7 @@ Git::Raw::Tag - Git tag class
 
 =head1 VERSION
 
-version 0.20
+version 0.21
 
 =head1 SYNOPSIS
 
@@ -60,9 +60,9 @@ as C<$repo-E<gt>lookup($id)> except that it only returns tags.
 Run C<$callback> for every tag in the repo. The callback receives a tag object.
 A non-zero return value stops the loop.
 
-=head2 delete( $repo, $name )
+=head2 delete( )
 
-Delete the tag with name C<$name>
+Delete the tag. The L<Git::Raw::Tag> object must not be accessed afterwards.
 
 =head2 id( )
 
