@@ -1493,7 +1493,7 @@ int git_reference_foreach(
 	/* list all the packed references first */
 	if (list_flags & GIT_REF_PACKED) {
 		const char *ref_name;
-		void *ref;
+		void *ref = NULL;
 		GIT_UNUSED(ref);
 
 		if (packed_load(repo) < 0)
@@ -1599,6 +1599,7 @@ static int ensure_segment_validity(const char *name)
 {
 	const char *current = name;
 	char prev = '\0';
+	int lock_len = strlen(GIT_FILELOCK_EXTENSION);
 
 	if (*current == '.')
 		return -1; /* Refname starts with "." */
@@ -1619,6 +1620,11 @@ static int ensure_segment_validity(const char *name)
 		prev = *current;
 	}
 
+	/* A refname component can not end with ".lock" */
+	if (current - name >= lock_len &&
+		!memcmp(current - lock_len, GIT_FILELOCK_EXTENSION, lock_len))
+			return -1;
+
 	return (int)(current - name);
 }
 
@@ -1691,6 +1697,10 @@ int git_reference__normalize_name(
 			segments_count++;
 		}
 
+		/* No empty segment is allowed when not normalizing */
+		if (segment_len == 0 && !normalize)
+			goto cleanup;
+		
 		if (current[segment_len] == '\0')
 			break;
 
@@ -1709,10 +1719,6 @@ int git_reference__normalize_name(
 	if (current[segment_len - 1] == '/')
 		goto cleanup;
 
-	/* A refname can not end with ".lock" */
-	if (!git__suffixcmp(name, GIT_FILELOCK_EXTENSION))
-		goto cleanup;
-
 	if ((segments_count == 1 ) && !(flags & GIT_REF_FORMAT_ALLOW_ONELEVEL))
 		goto cleanup;
 
