@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2012 the libgit2 contributors
+ * Copyright (C) the libgit2 contributors. All rights reserved.
  *
  * This file is part of libgit2, distributed under the GNU GPL v2 with
  * a Linking Exception. For full terms see the included COPYING file.
@@ -241,15 +241,15 @@ static const char *scan_for_previous_token(const char *buffer, const char *left_
 
 static int parse_time(git_time_t *time_out, const char *buffer)
 {
-	int time;
 	int error;
+	int64_t time;
 
 	if (*buffer == '+' || *buffer == '-') {
 		giterr_set(GITERR_INVALID, "Failed while parsing time. '%s' actually looks like a timezone offset.", buffer);
 		return -1;
 	}
 
-	error = git__strtol32(&time, buffer, &buffer, 10);
+	error = git__strtol64(&time, buffer, &buffer, 10);
 
 	if (!error)
 		*time_out = (git_time_t)time;
@@ -264,7 +264,7 @@ int git_signature__parse(git_signature *sig, const char **buffer_out,
 	const char *line_end, *name_end, *email_end, *tz_start, *time_start;
 	int error = 0;
 
-	memset(sig, 0x0, sizeof(git_signature));
+	memset(sig, 0, sizeof(git_signature));
 
 	if ((line_end = memchr(buffer, ender, buffer_end - buffer)) == NULL)
 		return signature_error("no newline given");
