@@ -7,9 +7,6 @@
 
 static git_repository *g_repo;
 static git_object *g_obj;
-static char g_orig_tz[16] = {0};
-
-
 
 /* Helpers */
 static void test_object_inrepo(const char *spec, const char *expected_oid, git_repository *repo)
@@ -37,19 +34,12 @@ static void test_object(const char *spec, const char *expected_oid)
 
 void test_refs_revparse__initialize(void)
 {
-	char *tz = cl_getenv("TZ");
-	if (tz)
-		strcpy(g_orig_tz, tz);
-	cl_setenv("TZ", "UTC");
-
 	cl_git_pass(git_repository_open(&g_repo, cl_fixture("testrepo.git")));
 }
 
 void test_refs_revparse__cleanup(void)
 {
 	git_repository_free(g_repo);
-	g_obj = NULL;
-	cl_setenv("TZ", g_orig_tz);
 }
 
 void test_refs_revparse__nonexistant_object(void)
@@ -59,13 +49,17 @@ void test_refs_revparse__nonexistant_object(void)
 	test_object("this-does-not-exist~2", NULL);
 }
 
+static void assert_invalid_spec(const char *invalid_spec)
+{
+	cl_assert_equal_i(
+		GIT_EINVALIDSPEC, git_revparse_single(&g_obj, g_repo, invalid_spec));
+}
+
 void test_refs_revparse__invalid_reference_name(void)
 {
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "this doesn't make sense"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "this doesn't make sense^1"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "this doesn't make sense~2"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, ""));
-
+	assert_invalid_spec("this doesn't make sense");
+	assert_invalid_spec("Inv@{id");
+	assert_invalid_spec("");
 }
 
 void test_refs_revparse__shas(void)
@@ -104,9 +98,11 @@ void test_refs_revparse__describe_output(void)
 
 void test_refs_revparse__nth_parent(void)
 {
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "be3563a^-1"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "^"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "be3563a^{tree}^"));
+	assert_invalid_spec("be3563a^-1");
+	assert_invalid_spec("^");
+	assert_invalid_spec("be3563a^{tree}^");
+	assert_invalid_spec("point_to_blob^{blob}^");
+	assert_invalid_spec("this doesn't make sense^1");
 
 	test_object("be3563a^1", "9fd738e8f7967c078dceed8190330fc8648ee56a");
 	test_object("be3563a^", "9fd738e8f7967c078dceed8190330fc8648ee56a");
@@ -133,8 +129,10 @@ void test_refs_revparse__not_tag(void)
 
 void test_refs_revparse__to_type(void)
 {
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "wrapped_tag^{blob}"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "wrapped_tag^{trip}"));
+	assert_invalid_spec("wrapped_tag^{trip}");
+	test_object("point_to_blob^{commit}", NULL);
+	cl_assert_equal_i(
+		GIT_EAMBIGUOUS, git_revparse_single(&g_obj, g_repo, "wrapped_tag^{blob}"));
 
 	test_object("wrapped_tag^{commit}", "a65fedf39aefe402d3bb6e24df4d4f5fe4547750");
 	test_object("wrapped_tag^{tree}", "944c0f6e4dfa41595e6eb3ceecdb14f50fe18162");
@@ -144,11 +142,15 @@ void test_refs_revparse__to_type(void)
 
 void test_refs_revparse__linear_history(void)
 {
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "~"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "foo~bar"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "master~bar"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "master~-1"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "master~0bar"));
+	assert_invalid_spec("~");
+	test_object("foo~bar", NULL);
+
+	assert_invalid_spec("master~bar");
+	assert_invalid_spec("master~-1");
+	assert_invalid_spec("master~0bar");
+	assert_invalid_spec("this doesn't make sense~2");
+	assert_invalid_spec("be3563a^{tree}~");
+	assert_invalid_spec("point_to_blob^{blob}~");
 
 	test_object("master~0", "a65fedf39aefe402d3bb6e24df4d4f5fe4547750");
 	test_object("master~1", "be3563ae3f795b2b4353bcce3a527ad0a4f7f644");
@@ -159,10 +161,10 @@ void test_refs_revparse__linear_history(void)
 
 void test_refs_revparse__chaining(void)
 {
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "master@{0}@{0}"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "@{u}@{-1}"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "@{-1}@{-1}"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "@{-3}@{0}"));
+	assert_invalid_spec("master@{0}@{0}");
+	assert_invalid_spec("@{u}@{-1}");
+	assert_invalid_spec("@{-1}@{-1}");
+	assert_invalid_spec("@{-3}@{0}");
 
 	test_object("master@{0}~1^1", "9fd738e8f7967c078dceed8190330fc8648ee56a");
 	test_object("@{u}@{0}", "be3563ae3f795b2b4353bcce3a527ad0a4f7f644");
@@ -178,8 +180,9 @@ void test_refs_revparse__chaining(void)
 
 void test_refs_revparse__upstream(void)
 {
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "e90810b@{u}"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "refs/tags/e90810b@{u}"));
+	assert_invalid_spec("e90810b@{u}");
+	assert_invalid_spec("refs/tags/e90810b@{u}");
+	test_object("refs/heads/e90810b@{u}", NULL);
 
 	test_object("master@{upstream}", "be3563ae3f795b2b4353bcce3a527ad0a4f7f644");
 	test_object("@{u}", "be3563ae3f795b2b4353bcce3a527ad0a4f7f644");
@@ -190,7 +193,7 @@ void test_refs_revparse__upstream(void)
 
 void test_refs_revparse__ordinal(void)
 {
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "master@{-2}"));
+	assert_invalid_spec("master@{-2}");
 	
 	/* TODO: make the test below actually fail
 	 * cl_git_fail(git_revparse_single(&g_obj, g_repo, "master@{1a}"));
@@ -212,9 +215,9 @@ void test_refs_revparse__ordinal(void)
 
 void test_refs_revparse__previous_head(void)
 {
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "@{-xyz}"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "@{-0}"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "@{-1b}"));
+	assert_invalid_spec("@{-xyz}");
+	assert_invalid_spec("@{-0}");
+	assert_invalid_spec("@{-1b}");
 
 	test_object("@{-42}", NULL);
 
@@ -271,9 +274,9 @@ void test_refs_revparse__reflog_of_a_ref_under_refs(void)
 
 void test_refs_revparse__revwalk(void)
 {
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "master^{/not found in any commit}"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "master^{/merge}"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, "master^{/((}"));
+	test_object("master^{/not found in any commit}", NULL);
+	test_object("master^{/merge}", NULL);
+	assert_invalid_spec("master^{/((}");
 
 	test_object("master^{/anoth}", "5b5b025afb0b4c913b4c338a42934a3863bf3644");
 	test_object("master^{/Merge}", "be3563ae3f795b2b4353bcce3a527ad0a4f7f644");
@@ -354,8 +357,9 @@ void test_refs_revparse__date(void)
 
 void test_refs_revparse__colon(void)
 {
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, ":/"));
-	cl_git_fail(git_revparse_single(&g_obj, g_repo, ":2:README"));
+	assert_invalid_spec(":/");
+	assert_invalid_spec("point_to_blob:readme.txt");
+	cl_git_fail(git_revparse_single(&g_obj, g_repo, ":2:README")); /* Not implemented  */
 
 	test_object(":/not found in any commit", NULL);
 	test_object("subtrees:ab/42.txt", NULL);
@@ -445,11 +449,8 @@ void test_refs_revparse__disambiguation(void)
 
 void test_refs_revparse__a_too_short_objectid_returns_EAMBIGUOUS(void)
 {
-	int result;
-	
-	result = git_revparse_single(&g_obj, g_repo, "e90");
-	
-	cl_assert_equal_i(GIT_EAMBIGUOUS, result);
+	cl_assert_equal_i(
+		GIT_EAMBIGUOUS, git_revparse_single(&g_obj, g_repo, "e90"));
 }
 
 void test_refs_revparse__issue_994(void)
@@ -485,3 +486,112 @@ void test_refs_revparse__issue_994(void)
 	git_reference_free(head);
 	cl_git_sandbox_cleanup();
 }
+
+/**
+ * $ git rev-parse blah-7-gc47800c
+ * c47800c7266a2be04c571c04d5a6614691ea99bd
+ * 
+ * $ git rev-parse HEAD~3
+ * 4a202b346bb0fb0db7eff3cffeb3c70babbd2045
+ * 
+ * $ git branch blah-7-gc47800c HEAD~3
+ * 
+ * $ git rev-parse blah-7-gc47800c
+ * 4a202b346bb0fb0db7eff3cffeb3c70babbd2045
+ */
+void test_refs_revparse__try_to_retrieve_branch_before_described_tag(void)
+{
+	git_repository *repo;
+	git_reference *branch;
+	git_object *target;
+	char sha[GIT_OID_HEXSZ + 1];
+
+	repo = cl_git_sandbox_init("testrepo.git");
+
+	test_object_inrepo("blah-7-gc47800c", "c47800c7266a2be04c571c04d5a6614691ea99bd", repo);
+
+	cl_git_pass(git_revparse_single(&target, repo, "HEAD~3"));
+	cl_git_pass(git_branch_create(&branch, repo, "blah-7-gc47800c", (git_commit *)target, 0));
+
+	git_oid_tostr(sha, GIT_OID_HEXSZ + 1, git_object_id(target));
+
+	test_object_inrepo("blah-7-gc47800c", sha, repo);
+
+	git_reference_free(branch);
+	git_object_free(target);
+	cl_git_sandbox_cleanup();
+}
+
+/**
+ * $ git rev-parse a65fedf39aefe402d3bb6e24df4d4f5fe4547750
+ * a65fedf39aefe402d3bb6e24df4d4f5fe4547750
+ * 
+ * $ git rev-parse HEAD~3
+ * 4a202b346bb0fb0db7eff3cffeb3c70babbd2045
+ * 
+ * $ git branch a65fedf39aefe402d3bb6e24df4d4f5fe4547750 HEAD~3
+ * 
+ * $ git rev-parse a65fedf39aefe402d3bb6e24df4d4f5fe4547750
+ * a65fedf39aefe402d3bb6e24df4d4f5fe4547750
+ * 
+ * $ git rev-parse heads/a65fedf39aefe402d3bb6e24df4d4f5fe4547750
+ * 4a202b346bb0fb0db7eff3cffeb3c70babbd2045
+ */
+void test_refs_revparse__try_to_retrieve_sha_before_branch(void)
+{
+	git_repository *repo;
+	git_reference *branch;
+	git_object *target;
+	char sha[GIT_OID_HEXSZ + 1];
+
+	repo = cl_git_sandbox_init("testrepo.git");
+
+	test_object_inrepo("a65fedf39aefe402d3bb6e24df4d4f5fe4547750", "a65fedf39aefe402d3bb6e24df4d4f5fe4547750", repo);
+
+	cl_git_pass(git_revparse_single(&target, repo, "HEAD~3"));
+	cl_git_pass(git_branch_create(&branch, repo, "a65fedf39aefe402d3bb6e24df4d4f5fe4547750", (git_commit *)target, 0));
+
+	git_oid_tostr(sha, GIT_OID_HEXSZ + 1, git_object_id(target));
+
+	test_object_inrepo("a65fedf39aefe402d3bb6e24df4d4f5fe4547750", "a65fedf39aefe402d3bb6e24df4d4f5fe4547750", repo);
+	test_object_inrepo("heads/a65fedf39aefe402d3bb6e24df4d4f5fe4547750", sha, repo);
+
+	git_reference_free(branch);
+	git_object_free(target);
+	cl_git_sandbox_cleanup();
+}
+
+/**
+ * $ git rev-parse c47800
+ * c47800c7266a2be04c571c04d5a6614691ea99bd
+ * 
+ * $ git rev-parse HEAD~3
+ * 4a202b346bb0fb0db7eff3cffeb3c70babbd2045
+ * 
+ * $ git branch c47800 HEAD~3
+ * 
+ * $ git rev-parse c47800
+ * 4a202b346bb0fb0db7eff3cffeb3c70babbd2045
+ */
+void test_refs_revparse__try_to_retrieve_branch_before_abbrev_sha(void)
+{
+	git_repository *repo;
+	git_reference *branch;
+	git_object *target;
+	char sha[GIT_OID_HEXSZ + 1];
+
+	repo = cl_git_sandbox_init("testrepo.git");
+
+	test_object_inrepo("c47800", "c47800c7266a2be04c571c04d5a6614691ea99bd", repo);
+
+	cl_git_pass(git_revparse_single(&target, repo, "HEAD~3"));
+	cl_git_pass(git_branch_create(&branch, repo, "c47800", (git_commit *)target, 0));
+
+	git_oid_tostr(sha, GIT_OID_HEXSZ + 1, git_object_id(target));
+
+	test_object_inrepo("c47800", sha, repo);
+
+	git_reference_free(branch);
+	git_object_free(target);
+	cl_git_sandbox_cleanup();
+}
